(* File generated from pplite.idl *)


(*
 This file is part of the APRON Library, released under GPL license.
 Please read the COPYING file packaged in the distribution.
*)

(** Polyhedra abstract domains (PPLite wrapper) *)

type loose
type strict
  (** Two flavors for convex polyhedra: loose or strict.

      Loose polyhedra cannot have strict inequality constraints like [x>0].
      Convex polyhedra are defined by the conjunction of a set of linear
      constraints of the form
      [a_0*x_0 + ... + a_n*x_n + b >= 0] or
      [a_0*x_0 + ... + a_n*x_n + b > 0]
      where [a_0, ..., a_n, b, c] are constants and [x_0, ..., x_n] variables.
  *)

type 'a t
(** Type of convex polyhedra, where ['a] is [loose] or [strict].

    Abstract values which are convex polyhedra have the type
    [loose t Apron.AbstractX.t] or [strict t Apron.AbstractX.t].
    Managers allocated by PPLite have the type ['a t Apron.Manager.t].
*)

(** Allocate a PPLite manager for loose convex polyhedra. *)
external manager_alloc_loose : unit -> loose t Apron.Manager.t
	= "camlidl_pplite_ap_pplite_manager_alloc_loose"

(** Allocate a PPLite manager for strict convex polyhedra. *)
external manager_alloc_strict : unit -> strict t Apron.Manager.t
	= "camlidl_pplite_ap_pplite_manager_alloc_strict"

(** Set the polyhedra kind for PPLite manager. *)
external manager_set_kind : 'a t Apron.Manager.t -> string -> unit
	= "camlidl_pplite_ap_pplite_manager_set_kind"

(** Get the polyhedra kind from PPLite manager. *)
external manager_get_kind : 'a t Apron.Manager.t -> string
	= "camlidl_pplite_ap_pplite_manager_get_kind"

(** Set the widening specification for PPLite manager.
   legal values are:
     - "safe": the Cousot&Cousot POPL77 specification, which does not
        assume the inclusion between arguments;
     - "risky": the Cousot&Cousot PLILP92 footnote 6 alternative
        specification, assuming the inclusion between arguments.
*)
external manager_set_widen_spec : 'a t Apron.Manager.t -> string -> unit
	= "camlidl_pplite_ap_pplite_manager_set_widen_spec"

(** Get the widening specification from PPLite manager. *)
external manager_get_widen_spec : 'a t Apron.Manager.t -> string
	= "camlidl_pplite_ap_pplite_manager_get_widen_spec"

(** {2 PPLite's abstract0 ad-hoc functions} *)
(** Splits abstract element on linear constraint *)
external abstract0_split : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> Apron.Lincons0.t -> bool -> bool -> 'a t Apron.Abstract0.t
	= "camlidl_pplite_ap_pplite_abstract0_split"

(** Returns true if abs is a powerset of polyhedra *)
external abstract0_is_disjunctive : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> bool
	= "camlidl_pplite_ap_pplite_abstract0_is_disjunctive"

(** Get the number of disjuncts of a polyhedra powerset. *)
external abstract0_num_disjuncts : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int
	= "camlidl_pplite_ap_pplite_abstract0_num_disjuncts"

(** Get the linear constraints of the n-th disjunct. *)
external abstract0_disj_to_lincons_array : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> Apron.Lincons0.t array
	= "camlidl_pplite_ap_pplite_abstract0_disj_to_lincons_array"

(** Get the tree constraints of the n-th disjunct. *)
external abstract0_disj_to_tcons_array : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> Tcons0.t array
	= "camlidl_pplite_ap_pplite_abstract0_disj_to_tcons_array"

(** Returns true if abs1 is geometrically contained in abs2 *)
external abstract0_geom_subseteq : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> 'a t Apron.Abstract0.t -> bool
	= "camlidl_pplite_ap_pplite_abstract0_geom_subseteq"

(** Modify abstract element to have n disjunct at most. *)
external abstract0_collapse : 'a t Apron.Manager.t -> 'a t Apron.Abstract0.t -> int -> unit
	= "camlidl_pplite_ap_pplite_abstract0_collapse"

(** {2 Type conversions} *)

val manager_is_pplite : 'a Apron.Manager.t -> bool
val manager_is_pplite_loose : 'a Apron.Manager.t -> bool
val manager_is_pplite_strict : 'a Apron.Manager.t -> bool
  (** Return [true] iff the argument manager is a PPLite manager *)
val manager_of_pplite : 'a t Apron.Manager.t -> 'b Apron.Manager.t
val manager_of_pplite_loose : loose t Apron.Manager.t -> 'a Apron.Manager.t
val manager_of_pplite_strict : strict t Apron.Manager.t -> 'a Apron.Manager.t
  (** Make a PPLite manager generic *)
val manager_to_pplite : 'a Apron.Manager.t -> 'b t Apron.Manager.t
val manager_to_pplite_loose : 'a Apron.Manager.t -> loose t Apron.Manager.t
val manager_to_pplite_strict : 'a Apron.Manager.t -> strict t Apron.Manager.t
  (** Instantiate the type of a PPLite manager.
      Raises [Failure] if the argument manager is not a PPLite manager *)

module Abstract0 : sig
  val is_pplite : 'a Apron.Abstract0.t -> bool
  val is_pplite_loose : 'a Apron.Abstract0.t -> bool
  val is_pplite_strict : 'a Apron.Abstract0.t -> bool
    (** Return [true] iff the argument manager is a PPLite value *)
  val of_pplite : 'a t Apron.Abstract0.t -> 'b Apron.Abstract0.t
  val of_pplite_loose : loose t Apron.Abstract0.t -> 'a Apron.Abstract0.t
  val of_pplite_strict : strict t Apron.Abstract0.t -> 'a Apron.Abstract0.t
    (** Make a PPLite value generic *)
  val to_pplite : 'a Apron.Abstract0.t -> 'b t Apron.Abstract0.t
  val to_pplite_loose : 'a Apron.Abstract0.t -> loose t Apron.Abstract0.t
  val to_pplite_strict : 'a Apron.Abstract0.t -> strict t Apron.Abstract0.t
    (** Instantiate the type of a PPLite value.
	Raises [Failure] if the argument manager is not a PPLite manager *)
end

module Abstract1 : sig
  val is_pplite : 'a Apron.Abstract1.t -> bool
  val is_pplite_loose : 'a Apron.Abstract1.t -> bool
  val is_pplite_strict : 'a Apron.Abstract1.t -> bool
    (** Return [true] iff the argument manager is a PPLite value *)
  val of_pplite : 'a t Apron.Abstract1.t -> 'b Apron.Abstract1.t
  val of_pplite_loose : loose t Apron.Abstract1.t -> 'a Apron.Abstract1.t
  val of_pplite_strict : strict t Apron.Abstract1.t -> 'a Apron.Abstract1.t
    (** Make a PPLite value generic *)
  val to_pplite : 'a Apron.Abstract1.t -> 'b t Apron.Abstract1.t
  val to_pplite_loose : 'a Apron.Abstract1.t -> loose t Apron.Abstract1.t
  val to_pplite_strict : 'a Apron.Abstract1.t -> strict t Apron.Abstract1.t
    (** Instantiate the type of a PPLite value.
	Raises [Failure] if the argument manager is not a PPLite manager *)
end


(**
{2 Compilation information}

See {!Introduction.compilation} for complete explanations.
We just show examples with the file [mlexample.ml].

Do not forget the [-cc "g++"] option: PPLite is a C++ library which requires
a C++ linker.

{3 Bytecode compilation}

{[ocamlc -cc "g++"-I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -o mlexample.byte \
  bigarray.cma gmp.cma apron.cma pplite.cma mlexample.ml]}

{[ocamlc -cc "g++" -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -make-runtime -o myrun \
  bigarray.cma gmp.cma apron.cma pplite.cma

ocamlc -cc "g++" -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -use-runtime myrun -o mlexample.byte \
  bigarray.cma gmp.cma apron.cma pplite.cma mlexample.ml ]}

{3 Native-code compilation}

{[ocamlopt -cc "g++" -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -o mlexample.opt \
  bigarray.cmxa gmp.cmxa apron.cmxa pplite.cmxa mlexample.ml ]}

{3 Without auto-linking feature}

{[ocamlopt -cc "g++" -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -noautolink -o mlexample.opt \
  bigarray.cmxa gmp.cmxa apron.cmxa pplite.cmxa mlexample.ml \
  -cclib "-L$MLGMPIDL_PREFIX/lib -L$APRON_PREFIX/lib -L$PPLITE_PREFIX/lib\
	  -lap_pplite_caml_debug -lap_pplite_debug -lpplite \
          -L$FLINT_PREFIX/lib -lflint \
	  -lapron_caml_debug -lapron_debug \
	  -lgmp_caml -L$MPFR_PREFIX/lib -lmpfr -L$GMP_PREFIX/lib -lgmp \
	  -L$CAMLIDL_PREFIX/lib/ocaml -lcamlidl \
	  -lbigarray" ]}

*)
